@@ -1,5 +1,5 @@
 
-#include "cheese-config.h"
+#include "config.h"
 
 #include <stdlib.h>
 
